package com.zego.ktvchorus.rtc;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;

import com.zego.ktvchorus.BaseApplication;
import com.zego.ktvchorus.model.RoomMusicInfo;
import com.zego.ktvchorus.rtc.callbacks.IZGKTVCopyrightedSongLoadCallback;
import com.zego.ktvchorus.rtc.callbacks.IZGKTVGetLyricCallback;
import com.zego.ktvchorus.rtc.callbacks.IZGKTVRequestMusicCallback;
import com.zego.ktvchorus.utils.T;
import com.zego.ktvchorus.utils.ThreadHelper;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import im.zego.lyricview.model.ZGKTVLyric;
import im.zego.zegoexpress.callback.IZegoCopyrightedMusicDownloadCallback;

/**
 * 文件下载管理器
 */
public class FileDownloadManager {

    private static final String TAG = "FileDownloadManager";


    private volatile static FileDownloadManager mInstance;

    private BaseFileDownloadDelivery downloadDelivery;
    /**
     * context - listeners map
     */
    private ConcurrentHashMap<Context, ArrayList<FileDownloadStateListener>> contextMap;

    private ConcurrentHashMap<String, String> songKeyMap;

    private ArrayList<String> dowloadedSongIdList = new ArrayList<>();


    private FileDownloadManager() {

        downloadDelivery = new DefaultFileDownloadDelivery();
        contextMap = new ConcurrentHashMap<>();
        songKeyMap = new ConcurrentHashMap<>();

        ((Application) (BaseApplication.sApplication)).registerActivityLifecycleCallbacks(new Application.ActivityLifecycleCallbacks() {
            @Override
            public void onActivityCreated(Activity activity, Bundle savedInstanceState) {

            }

            @Override
            public void onActivityStarted(Activity activity) {

            }

            @Override
            public void onActivityResumed(Activity activity) {

            }

            @Override
            public void onActivityPaused(Activity activity) {

            }

            @Override
            public void onActivityStopped(Activity activity) {

            }

            @Override
            public void onActivitySaveInstanceState(Activity activity, Bundle outState) {

            }

            @Override
            public void onActivityDestroyed(Activity activity) {
                unRegisterDownloadListener(activity);
            }
        });

        onLoadUpdate();
    }

    public static FileDownloadManager getInstance() {
        if (mInstance == null) {
            synchronized (FileDownloadManager.class) {
                if (mInstance == null) {
                    mInstance = new FileDownloadManager();
                }
            }
        }
        return mInstance;
    }

    /**
     * 将回调加入Delivery
     * 只有通过这个方法注册的回调才会使用Delivery
     * 防止普通用法download批量后导致Delivery中数据过多而回调缓慢
     * 必须先启动下载，再来注册，否则不会添加到Delivery中
     *
     * @param context                   上下文
     * @param tag                       标记
     * @param fileDownloadStateListener 回调
     */
    public void registerDownloadListener(Context context, String tag, FileDownloadStateListener fileDownloadStateListener) {
        if (fileDownloadStateListener != null) {
            if (context != null) {
                ArrayList<FileDownloadStateListener> listeners = contextMap.get(context);
                if (listeners == null) {
                    listeners = new ArrayList<>();
                    contextMap.put(context, listeners);
                }
                listeners.add(fileDownloadStateListener);
            }
            registerDownloadListener(tag, fileDownloadStateListener);
        }
    }

    /**
     * 这里用于没有绑定UI的用法，如果有绑定UI，请使用Context的方式
     *
     * @param tag                       标记
     * @param fileDownloadStateListener 回调
     */
    public void registerDownloadListener(String tag, FileDownloadStateListener fileDownloadStateListener) {
        if (fileDownloadStateListener != null) {
            downloadDelivery.addFileDownloadListener(tag, fileDownloadStateListener);
        }
    }

    /**
     * 移除回调
     * 这里用于没有绑定UI的用法，如果有绑定UI，请使用Context的方式
     *
     * @param tag
     */
    public void unRegisterDownloadListener(String tag) {
        downloadDelivery.removeFileDownloadListener(tag);
    }

    /**
     * 移除回调
     *
     * @param listener
     */
    public void unRegisterDownloadListener(FileDownloadStateListener listener) {
        downloadDelivery.removeFileDownloadListener(listener);
    }

    /**
     * 移除回调
     *
     * @param context
     */
    public void unRegisterDownloadListener(Context context) {
        ArrayList<FileDownloadStateListener> tags = contextMap.get(context);
        if (tags != null) {
            for (FileDownloadStateListener tag : tags) {
                unRegisterDownloadListener(tag);
            }
        }
        contextMap.remove(context);
    }

    public void clearData() {
        songKeyMap.clear();
        dowloadedSongIdList.clear();
    }

    /**
     * 判断某个tag是否在下载中
     *
     * @param tag
     * @return
     */
    public boolean isDownloading(String tag) {
        if (TextUtils.isEmpty(tag)) {
            return false;
        }
        return songKeyMap.containsKey(tag);
    }

    public void downloadFile(String songId, RoomMusicInfo roomMusicInfo) {
        downloadFile(songId, roomMusicInfo, false);
    }

    public void downloadFile(String songId, RoomMusicInfo roomMusicInfo, boolean needSwitchMenu) {
        for (ZGKTVCopyrightedSong song : KTVRoomOrderSongManager.getInstance().getOrderList()) {
            if (song.getSongID().equals(songId)) {
                // 已经下载过了 移动到最后一个
                KTVRoomOrderSongManager.getInstance().getOrderList().remove(song);
                KTVRoomOrderSongManager.getInstance().addOrderSong(song, needSwitchMenu);
                return;
            }
        }

        songKeyMap.put(songId, songId);
        ZGKTVCopyrightedMusicManager.getInstance().requestAccompaniment(songId, new IZGKTVRequestMusicCallback() {
            @Override
            public void onRequestMusicCallback(int errorCode, String message, ZGKTVCopyrightedSong song) {
                Log.d(TAG, songId + "----requestAccompaniment---" + errorCode);
                if (errorCode == 0) {
                    ZGKTVPlayerManager.getInstance().load(songId, new IZegoCopyrightedMusicDownloadCallback() {
                        @Override
                        public void onDownloadCallback(int errorCode) {
                            if (ZGKTVCopyrightedMusicManager.isMusicSDKErrorCode(errorCode)) {
                                T.show(ZGKTVCopyrightedMusicManager.getErrorInfo(errorCode));
                            }
                        }
                    });
                    if (roomMusicInfo != null) {
                        song.setSongID(roomMusicInfo.getSongID());
                        song.setSongName(roomMusicInfo.getSongName());
                        song.setSingerName(roomMusicInfo.getSingerName());
                        song.setAlbumImageUri(roomMusicInfo.getAlbumImg());
                        if (roomMusicInfo.getKrcToken() != null) {
                            song.setKrcToken(roomMusicInfo.getKrcToken());
                        }
                    }
                    KTVRoomOrderSongManager.getInstance().addOrderSong(song, needSwitchMenu);
                } else if (ZGKTVCopyrightedMusicManager.isMusicSDKErrorCode(errorCode)) {
                    T.show(ZGKTVCopyrightedMusicManager.getErrorInfo(errorCode));
                    listenerError(songId);
                } else {
                    listenerError(songId);
                }
            }
        });
    }


    /**
     * 下载监听回调
     */
    private void onLoadUpdate() {
        ZGKTVSongLoadManager.getInstance().setLoadCallback(new IZGKTVCopyrightedSongLoadCallback() {
            @Override
            public void onLoadUpdate(String songID, ZGKTVSongLoadState songLoadState, float progressRate) {

                Log.d(TAG, songID + "---songID--" + progressRate + "---progressRate---" + songLoadState.value() + "---songLoadState-");

                switch (songLoadState) {
                    case LOADING:
                        //下载中
                        listenerProgress(progressRate, songID);
                        break;
                    case LOAD_COMPLETE:
                        //下载完成
                        if (ZGKTVLyricManager.getInstance().getLyric(songID) == null) {
                            ZGKTVCopyrightedMusicManager.getInstance().getKrcLyricForSongID(songID, new IZGKTVGetLyricCallback() {
                                @Override
                                public void onGetLyricCallback(int errorCode, String message, ZGKTVLyric song) {
                                    if (errorCode == 0) {
                                        // KTVRoomOrderSongManager.getInstance().songDownloadReport(LoginInfoManager.getInstance().getLoginInfo().getUid(), songID);
                                    } else if (ZGKTVCopyrightedMusicManager.isMusicSDKErrorCode(errorCode)) {
                                        //                                        T.show(ZGKTVCopyrightedMusicManager.getErrorInfo(errorCode));
                                    }
                                }
                            });
                        } else {
                            // KTVRoomOrderSongManager.getInstance().songDownloadReport(LoginInfoManager.getInstance().getLoginInfo().getUid(), songID);
                        }
                        listenerCompleted(songID);
                        break;
                    case LOAD_ERROR:
                        //加载异常
                        listenerError(songID);
                        break;
                    default:
                        break;

                }

            }

        });

    }


    /**
     * 下载进度的回调
     */
    private void listenerProgress(float progress, String songId) {
        ThreadHelper.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                ArrayList<FileDownloadStateListener> listeners = new ArrayList<>();
                if (songKeyMap.containsKey(songId)) {
                    List<FileDownloadStateListener> tagListeners = downloadDelivery.getFileDownloadListeners(songId);
                    if (tagListeners != null && !tagListeners.isEmpty()) {
                        for (FileDownloadStateListener listener : tagListeners) {
                            listeners.add(listener);
                        }
                    }
                }

                for (FileDownloadStateListener listener : listeners) {
                    if (listener != null) {
                        listener.onDownloadState(ZGKTVSongLoadState.LOADING);
                        listener.onDownloadProgress(progress);
                    }
                }
            }
        });
    }


    /**
     * 下载完成回调
     */
    private void listenerCompleted(String songId) {
        ThreadHelper.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                ArrayList<FileDownloadStateListener> listeners = new ArrayList<>();
                if (songKeyMap.containsKey(songId)) {
                    List<FileDownloadStateListener> tagListeners = downloadDelivery.getFileDownloadListeners(songId);
                    if (tagListeners != null && !tagListeners.isEmpty()) {
                        for (FileDownloadStateListener listener : tagListeners) {
                            listeners.add(listener);
                        }
                    }
                }

                for (FileDownloadStateListener listener : listeners) {
                    if (listener != null) {
                        listener.onDownloadState(ZGKTVSongLoadState.LOAD_COMPLETE);
                    }
                }
                Log.d(TAG, songId + "----listenerCompleted---" + songId);
                //下载成功，移除回调
                downloadDelivery.removeFileDownloadListener(songId);
                songKeyMap.remove(songId);
                dowloadedSongIdList.add(songId);
            }
        });
    }


    /**
     * 下载出错回调
     *
     * @param songId
     */
    private void listenerError(String songId) {
        ThreadHelper.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                ArrayList<FileDownloadStateListener> listeners = new ArrayList<>();
                if (songKeyMap.containsKey(songId)) {
                    List<FileDownloadStateListener> tagListeners = downloadDelivery.getFileDownloadListeners(songId);
                    if (tagListeners != null && !tagListeners.isEmpty()) {
                        for (FileDownloadStateListener listener : tagListeners) {
                            listeners.add(listener);
                        }
                    }
                }

                for (FileDownloadStateListener listener : listeners) {
                    if (listener != null) {
                        listener.onDownloadState(ZGKTVSongLoadState.LOAD_ERROR);
                    }
                }
                Log.d(TAG, songId + "----listenerError---" + songId);
                //下载失败，移除回调
                downloadDelivery.removeFileDownloadListener(songId);
                songKeyMap.remove(songId);
            }
        });
    }

    public ArrayList<String> getDowloadedSongIdList() {
        return dowloadedSongIdList;
    }
}